/*
 * 파일명: kakao-login.js
 * 위치: /wp-content/plugins/wp-kakao-login/assets/js/
 * 기능: 카카오 로그인 자바스크립트
 * 작성일: 2025-05-01
 * 수정일: 2025-05-01 - 별도 로그인 페이지 리다이렉트 추가
 */

(function($) {
    'use strict';
    
    // ===================================
    // 초기 설정
    // ===================================
    function init() {
        console.log('Kakao Login Script Initialized');
        
        // 카카오 SDK 초기화
        if (typeof Kakao !== 'undefined' && wpKakaoLogin.apiKey) {
            console.log('Kakao SDK Found, initializing with: ' + wpKakaoLogin.apiKey);
            Kakao.init(wpKakaoLogin.apiKey);
            
            // 로그인 버튼 이벤트 바인딩 - 모든 카카오 로그인 버튼에 적용
            $(document).on('click', '#kakao-login-btn, .kakao-login-btn', handleKakaoButtonClick);
        } else {
            console.error('Kakao SDK not loaded or API Key missing');
            if (typeof Kakao === 'undefined') {
                console.error('Kakao SDK is undefined');
            }
            if (!wpKakaoLogin.apiKey) {
                console.error('API Key is missing');
            }
        }
    }
    
    // ===================================
    // 카카오 로그인 버튼 클릭 처리
    // ===================================
    function handleKakaoButtonClick(e) {
        e.preventDefault();
        console.log('Kakao Login Button Clicked');
        
        // 사이트 URL 추출 (홈 URL 기준)
        var site_url = wpKakaoLogin.redirectUri.split('/kakao-callback')[0];
        
        // 별도 카카오 로그인 전용 페이지로 리다이렉트
        console.log('Redirecting to dedicated Kakao login page');
        window.location.href = site_url + '/kakao-login';
        
        // 아래 직접 로그인 처리 코드는 더 이상 사용하지 않음
        // 별도 페이지에서 처리하도록 변경됨
    }
    
    // ===================================
    // 카카오 로그인 처리 (콜백 페이지에서 사용)
    // ===================================
    function handleKakaoLogin() {
        console.log('Direct Kakao Login Flow - This is kept for reference but not used anymore');
        
        if (typeof Kakao === 'undefined') {
            console.error('Kakao SDK가 로드되지 않았습니다.');
            alert('카카오 로그인을 초기화하는 중 오류가 발생했습니다. 페이지를 새로고침하고 다시 시도해주세요.');
            return;
        }
        
        try {
            console.log('Attempting Kakao Login...');
            Kakao.Auth.login({
                success: function(authObj) {
                    // 로그인 성공 처리
                    console.log('Kakao Login Success:', authObj);
                    processKakaoLogin(authObj.access_token);
                },
                fail: function(err) {
                    console.error('카카오 로그인 실패:', err);
                    alert('카카오 로그인 중 오류가 발생했습니다: ' + err.message);
                }
            });
        } catch (e) {
            console.error('카카오 로그인 오류:', e);
            alert('카카오 로그인 처리 중 오류가 발생했습니다. 페이지를 새로고침하고 다시 시도해주세요.');
        }
    }
    
    // ===================================
    // 서버 측 로그인 처리 (콜백 페이지에서 사용)
    // ===================================
    function processKakaoLogin(accessToken) {
        console.log('Processing Kakao Login with token');
        
        if (!accessToken) {
            console.error('액세스 토큰이 없습니다.');
            alert('인증 토큰을 받지 못했습니다. 다시 시도해주세요.');
            return;
        }
        
        // AJAX 요청으로 서버 측 로그인 처리
        $.ajax({
            url: wpKakaoLogin.ajaxUrl,
            type: 'POST',
            data: {
                action: 'kakao_login',
                nonce: wpKakaoLogin.nonce,
                access_token: accessToken
            },
            dataType: 'json',
            success: function(response) {
                console.log('Server response:', response);
                if (response.success) {
                    // 리다이렉트
                    window.location.href = response.redirect;
                } else {
                    // 오류 메시지 표시
                    alert(response.message || '로그인 중 오류가 발생했습니다.');
                }
            },
            error: function(xhr, status, error) {
                console.error('AJAX 요청 오류:', error);
                console.error('상태:', status);
                console.error('응답:', xhr.responseText);
                alert('로그인 요청 중 오류가 발생했습니다. 다시 시도해주세요.');
            }
        });
    }
    
    // ===================================
    // 콜백 처리 - URL에서 code 파라미터 확인
    // ===================================
    function checkCallbackCode() {
        // 현재 페이지가 콜백 페이지인지 확인 (URL에 /kakao-callback이 포함되어 있는지)
        if (window.location.pathname.indexOf('kakao-callback') !== -1) {
            console.log('On callback page, checking for code parameter');
            
            // URL에서 code 파라미터 추출
            var urlParams = new URLSearchParams(window.location.search);
            var code = urlParams.get('code');
            
            if (code) {
                console.log('Code parameter found, processing callback');
                // 코드가 있으면 서버에 전송하여 처리
                $.ajax({
                    url: wpKakaoLogin.ajaxUrl,
                    type: 'POST',
                    data: {
                        action: 'kakao_callback',
                        nonce: wpKakaoLogin.nonce,
                        code: code
                    },
                    dataType: 'json',
                    success: function(response) {
                        console.log('Callback response:', response);
                        if (response.success) {
                            // 리다이렉트
                            window.location.href = response.redirect;
                        } else {
                            // 오류 메시지 표시
                            alert(response.message || '로그인 처리 중 오류가 발생했습니다.');
                            // 홈페이지로 리다이렉트
                            window.location.href = wpKakaoLogin.redirectUri.split('/kakao-callback')[0];
                        }
                    },
                    error: function(xhr, status, error) {
                        console.error('콜백 처리 오류:', error);
                        alert('로그인 콜백 처리 중 오류가 발생했습니다. 다시 시도해주세요.');
                        // 홈페이지로 리다이렉트
                        window.location.href = wpKakaoLogin.redirectUri.split('/kakao-callback')[0];
                    }
                });
            }
        }
    }
    
    // DOM 준비 후 초기화
    $(document).ready(function() {
        console.log('Document ready, initializing Kakao Login');
        init();
        
        // 콜백 코드 확인 (콜백 페이지에서만 동작)
        checkCallbackCode();
    });
    
})(jQuery);
