<?php
/*
 * 파일명: direct-kakao-callback.php
 * 위치: /wp-content/plugins/wp-kakao-login/
 * 기능: 카카오 로그인 콜백 직접 처리
 * 작성일: 2025-05-01
 * 수정일: 2025-05-02
 */

// ===================================
// 워드프레스 로드 및 필요한 파일 포함
// ===================================
// 워드프레스 로드
require_once(dirname(dirname(dirname(dirname(__FILE__)))) . '/wp-load.php');

// 필요한 파일 로드
require_once(plugin_dir_path(__FILE__) . 'includes/functions.php');
require_once(plugin_dir_path(__FILE__) . 'includes/class-kakao-login.php');

// 로그 기록
wp_kakao_login_debug_log('Kakao Callback Direct Access: ' . print_r($_GET, true));

// GET 파라미터 확인
if (!isset($_GET['code'])) {
    // 코드가 없는 경우 오류 처리
    wp_kakao_login_debug_log('Kakao Callback Error: No code parameter found');
    wp_redirect(home_url());
    exit;
}

// authorization code 가져오기
$code = sanitize_text_field($_GET['code']);
wp_kakao_login_debug_log('Kakao Authorization Code: ' . $code);

// state 파라미터가 있으면 저장 (로그인 후 리다이렉트 URL)
$state = isset($_GET['state']) ? esc_url_raw($_GET['state']) : '';
if (empty($state) || strpos($state, 'wp-login.php') !== false) {
    // 빈 state 또는 wp-login.php가 포함된 경우 항상 홈페이지로 리다이렉트
    $state = home_url();
}
wp_kakao_login_debug_log('Redirect State: ' . $state);

// 옵션 가져오기
$options = get_option('wp_kakao_login_options', array());
$client_id = isset($options['kakao_client_id']) ? $options['kakao_client_id'] : '';
$client_secret = isset($options['kakao_client_secret']) ? $options['kakao_client_secret'] : '';

// 플러그인 URL 구하기 및 리다이렉트 URI 설정
$plugin_url = plugin_dir_url(__FILE__);
$redirect_uri = $plugin_url . 'direct-kakao-callback.php';

// API 키가 없는 경우 오류 처리
if (empty($client_id)) {
    wp_kakao_login_debug_log('Kakao API Key is missing');
    wp_redirect(home_url('?login_error=api_key'));
    exit;
}

// 토큰 요청 URL 및 파라미터 설정
$token_url = 'https://kauth.kakao.com/oauth/token';
$args = array(
    'body' => array(
        'grant_type' => 'authorization_code',
        'client_id' => $client_id,
        'redirect_uri' => $redirect_uri,
        'code' => $code,
    ),
    'timeout' => 30
);

// 클라이언트 시크릿이 있는 경우 추가
if (!empty($client_secret)) {
    $args['body']['client_secret'] = $client_secret;
}

wp_kakao_login_debug_log('Kakao Token Request: ' . json_encode($args));

// API 요청
$response = wp_remote_post($token_url, $args);

// 요청 실패 시 오류 처리
if (is_wp_error($response)) {
    wp_kakao_login_debug_log('Kakao Token Request Error: ' . $response->get_error_message());
    wp_redirect(home_url('?login_error=token'));
    exit;
}

// 응답 코드 확인
$response_code = wp_remote_retrieve_response_code($response);
if ($response_code !== 200) {
    $body = wp_remote_retrieve_body($response);
    wp_kakao_login_debug_log('Kakao Token API Error Response: ' . $body);
    wp_redirect(home_url('?login_error=token'));
    exit;
}

// 응답 처리
$body = wp_remote_retrieve_body($response);
$data = json_decode($body, true);

if (empty($data) || !isset($data['access_token'])) {
    wp_kakao_login_debug_log('Kakao Token API Invalid Response: ' . $body);
    wp_redirect(home_url('?login_error=token'));
    exit;
}

// 액세스 토큰 가져오기
$access_token = $data['access_token'];
wp_kakao_login_debug_log('Kakao Access Token Retrieved: ' . substr($access_token, 0, 10) . '...');

// 사용자 정보 요청
$user_info = get_kakao_user_info($access_token);

if (is_wp_error($user_info)) {
    wp_kakao_login_debug_log('Kakao User Info Error: ' . $user_info->get_error_message());
    wp_redirect(home_url('?login_error=user_info'));
    exit;
}

// 사용자 인증/생성 처리
$result = authenticate_kakao_user($user_info);

if (is_wp_error($result)) {
    wp_kakao_login_debug_log('Kakao Authentication Error: ' . $result->get_error_message());
    wp_redirect(home_url('?login_error=auth'));
    exit;
}

// 로그인 성공 시 리다이렉트
// state 파라미터가 비어있는지 다시 확인
if (empty($state) || $state == 'null') {
    wp_kakao_login_debug_log('Empty state parameter, redirecting to home');
    $state = home_url();
}

wp_kakao_login_debug_log('Redirecting to: ' . $state);
wp_redirect($state);
exit;



/**
 * 카카오 사용자 정보 가져오기
 *
 * @param string $access_token 액세스 토큰
 * @return array|WP_Error 사용자 정보 또는 에러
 */
function get_kakao_user_info($access_token) {
    // 요청 URL 및 헤더 설정
    $api_url = 'https://kapi.kakao.com/v2/user/me';
    $args = array(
        'headers' => array(
            'Authorization' => 'Bearer ' . $access_token,
            'Content-Type' => 'application/x-www-form-urlencoded;charset=utf-8'
        ),
        'timeout' => 30
    );
    
    // API 요청
    $response = wp_remote_get($api_url, $args);
    
    // 요청 실패 시 오류 반환
    if (is_wp_error($response)) {
        return $response;
    }
    
    // 응답 코드 확인
    $response_code = wp_remote_retrieve_response_code($response);
    if ($response_code !== 200) {
        $body = wp_remote_retrieve_body($response);
        wp_kakao_login_debug_log('Kakao User Info API Error Response: ' . $body);
        return new WP_Error('user_info_request_failed', '사용자 정보 요청 실패: ' . $response_code);
    }
    
    // 응답 처리
    $body = wp_remote_retrieve_body($response);
    $data = json_decode($body, true);
    
    if (empty($data) || !isset($data['id'])) {
        wp_kakao_login_debug_log('Kakao User Info API Invalid Response: ' . $body);
        return new WP_Error('invalid_user_info_response', '유효하지 않은 사용자 정보 응답');
    }
    
    return $data;
}




/**
 * 카카오 사용자 인증 또는 생성
 *
 * @param array $user_info 카카오 사용자 정보
 * @return int|WP_Error 사용자 ID 또는 에러
 */
function authenticate_kakao_user($user_info) {
    // 카카오 ID 확인
    $kakao_id = isset($user_info['id']) ? $user_info['id'] : '';
    if (empty($kakao_id)) {
        return new WP_Error('missing_kakao_id', '카카오 ID가 없습니다.');
    }
    
    // 카카오 계정 정보
    $kakao_nickname = isset($user_info['properties']['nickname']) ? $user_info['properties']['nickname'] : '';
    $kakao_email = isset($user_info['kakao_account']['email']) ? $user_info['kakao_account']['email'] : '';
    
    // 로그 기록
    wp_kakao_login_debug_log('Kakao User Info: ID=' . $kakao_id . ', Nickname=' . $kakao_nickname . ', Email=' . $kakao_email);
    
    // 카카오 ID로 사용자 검색
    $users = get_users(array(
        'meta_key' => 'kakao_id',
        'meta_value' => $kakao_id,
        'number' => 1,
        'count_total' => false
    ));
    
    $user = !empty($users) ? $users[0] : false;
    
    // 기존 사용자가 있는 경우 로그인 처리
    if ($user) {
        wp_kakao_login_debug_log('Existing Kakao User Found: ' . $user->ID);
        wp_set_auth_cookie($user->ID, true);
        do_action('wp_login', $user->user_login, $user);
        return $user->ID;
    }
    
    // 이메일이 있는 경우 해당 이메일로 사용자 검색
    if (!empty($kakao_email)) {
        $user = get_user_by('email', $kakao_email);
        if ($user) {
            wp_kakao_login_debug_log('Existing User Found By Email: ' . $user->ID);
            // 기존 사용자에게 카카오 ID 연결
            update_user_meta($user->ID, 'kakao_id', $kakao_id);
            wp_set_auth_cookie($user->ID, true);
            do_action('wp_login', $user->user_login, $user);
            return $user->ID;
        }
    }
    
    // 새 사용자 생성
    return create_new_kakao_user($kakao_id, $kakao_nickname, $kakao_email);
}



/**
 * 새 카카오 사용자 생성
 *
 * @param string $kakao_id 카카오 ID
 * @param string $nickname 사용자 닉네임
 * @param string $email 사용자 이메일
 * @return int|WP_Error 사용자 ID 또는 에러
 */
function create_new_kakao_user($kakao_id, $nickname, $email) {
    // 이메일이 없는 경우 임시 이메일 생성
    if (empty($email)) {
        $email = 'kakao_' . $kakao_id . '@kakao.user';
    }
    
    // 사용자명 생성 (닉네임 + 랜덤 숫자)
    $username = sanitize_user($nickname, true);
    if (empty($username)) {
        $username = 'kakao_user';
    }
    
    // 중복 방지
    if (username_exists($username)) {
        $username .= '_' . substr($kakao_id, -6);
    }
    
    // 랜덤 비밀번호 생성
    $password = wp_generate_password(16, true, true);
    
    // 사용자 생성
    $user_id = wp_create_user($username, $password, $email);
    
    if (is_wp_error($user_id)) {
        wp_kakao_login_debug_log('Kakao User Creation Error: ' . $user_id->get_error_message());
        return $user_id;
    }
    
    // 사용자 메타 데이터 업데이트
    update_user_meta($user_id, 'kakao_id', $kakao_id);
    update_user_meta($user_id, 'nickname', $nickname);
    
    // 사용자 역할 설정
    $user = new WP_User($user_id);
    $user->set_role('subscriber');
    
    wp_kakao_login_debug_log('New Kakao User Created: ' . $user_id);
    
    // 인증 쿠키 설정
    wp_set_auth_cookie($user_id, true);
    do_action('wp_login', $username, $user);
    
    return $user_id;
}