<?php
/*
 * 파일명: class-kakao-admin.php
 * 위치: /wp-content/plugins/wp-kakao-login/includes/admin/
 * 기능: 카카오 로그인 관리자 설정 페이지
 * 작성일: 2025-05-01
 * 수정일: 2025-05-01
 */

// 직접 접근 방지
if (!defined('ABSPATH')) {
    exit;
}

// ===================================
// 카카오 로그인 관리자 클래스
// ===================================
class WP_Kakao_Admin {
    /**
     * 생성자
     */
    public function __construct() {
        // 관리자 메뉴 추가
        add_action('admin_menu', array($this, 'add_admin_menu'));
        
        // 관리자 스타일 및 스크립트 등록
        add_action('admin_enqueue_scripts', array($this, 'enqueue_admin_scripts'));
        
        // 설정 등록
        add_action('admin_init', array($this, 'register_settings'));
    }
    
    /**
     * 관리자 메뉴 추가
     */
    public function add_admin_menu() {
        add_options_page(
            __('카카오 로그인 설정', 'wp-kakao-login'),  // 페이지 제목
            __('카카오 로그인', 'wp-kakao-login'),       // 메뉴 제목
            'manage_options',                            // 필요 권한
            'wp-kakao-login',                           // 메뉴 슬러그
            array($this, 'render_admin_page')           // 콜백 함수
        );
    }
    
    /**
     * 관리자 스타일 및 스크립트 등록
     * 
     * @param string $hook_suffix 현재 관리자 페이지 훅 접미사
     */
    public function enqueue_admin_scripts($hook_suffix) {
        // 카카오 로그인 설정 페이지인 경우에만 스타일 및 스크립트 로드
        if ($hook_suffix != 'settings_page_wp-kakao-login') {
            return;
        }
        
        // Dashicons 등록 (WordPress 아이콘 폰트)
        wp_enqueue_style('dashicons');
        
        // jQuery 의존성 확인
        wp_enqueue_script('jquery');
        
        // 관리자 CSS 등록
        wp_enqueue_style(
            'wp-kakao-login-admin',
            WP_KAKAO_LOGIN_PLUGIN_URL . 'assets/css/admin.css',
            array(),
            WP_KAKAO_LOGIN_VERSION
        );
        
        // 관리자 JavaScript 등록
        wp_enqueue_script(
            'wp-kakao-login-admin',
            WP_KAKAO_LOGIN_PLUGIN_URL . 'assets/js/admin.js',
            array('jquery'),
            WP_KAKAO_LOGIN_VERSION,
            true
        );
        
        // 스크립트 데이터 전달
        wp_localize_script('wp-kakao-login-admin', 'wpKakaoLoginAdmin', array(
            'ajaxUrl' => admin_url('admin-ajax.php'),
            'nonce' => wp_create_nonce('wp_kakao_login_admin_nonce'),
            'homeUrl' => home_url()
        ));
    }
    
    /**
     * 설정 등록
     */
    public function register_settings() {
        // 설정 그룹 등록
        register_setting(
            'wp_kakao_login_options',          // 옵션 그룹
            'wp_kakao_login_options',          // 옵션 이름
            array($this, 'sanitize_options')  // 콜백 함수
        );
        
        // 기본 설정 섹션 추가
        add_settings_section(
            'wp_kakao_login_general',          // 섹션 ID
            __('기본 설정', 'wp-kakao-login'),  // 섹션 제목
            array($this, 'render_general_section'), // 콜백 함수
            'wp_kakao_login'                  // 페이지 슬러그
        );
        
        // API 키 필드 추가
        add_settings_field(
            'kakao_client_id',                // 필드 ID
            __('카카오 REST API 키', 'wp-kakao-login'), // 필드 제목
            array($this, 'render_client_id_field'), // 콜백 함수
            'wp_kakao_login',                 // 페이지 슬러그
            'wp_kakao_login_general'          // 섹션 ID
        );
        
        // 클라이언트 시크릿 필드 추가
        add_settings_field(
            'kakao_client_secret',           // 필드 ID
            __('카카오 Client Secret', 'wp-kakao-login'), // 필드 제목
            array($this, 'render_client_secret_field'), // 콜백 함수
            'wp_kakao_login',                // 페이지 슬러그
            'wp_kakao_login_general'         // 섹션 ID
        );
        
        // 리다이렉트 URI 필드 추가
        add_settings_field(
            'kakao_redirect_uri',            // 필드 ID
            __('리다이렉트 URI', 'wp-kakao-login'), // 필드 제목
            array($this, 'render_redirect_uri_field'), // 콜백 함수
            'wp_kakao_login',                // 페이지 슬러그
            'wp_kakao_login_general'         // 섹션 ID
        );
        
        // 버튼 텍스트 필드 추가
        add_settings_field(
            'login_button_text',             // 필드 ID
            __('로그인 버튼 텍스트', 'wp-kakao-login'), // 필드 제목
            array($this, 'render_button_text_field'), // 콜백 함수
            'wp_kakao_login',                // 페이지 슬러그
            'wp_kakao_login_general'         // 섹션 ID
        );
        
        // 고급 설정 섹션 추가
        add_settings_section(
            'wp_kakao_login_advanced',        // 섹션 ID
            __('고급 설정', 'wp-kakao-login'),  // 섹션 제목
            array($this, 'render_advanced_section'), // 콜백 함수
            'wp_kakao_login'                 // 페이지 슬러그
        );
        
        // 디버그 모드 필드 추가
        add_settings_field(
            'debug_mode',                    // 필드 ID
            __('디버그 모드', 'wp-kakao-login'), // 필드 제목
            array($this, 'render_debug_mode_field'), // 콜백 함수
            'wp_kakao_login',                // 페이지 슬러그
            'wp_kakao_login_advanced'        // 섹션 ID
        );
    }
    
    /**
     * 옵션 정제
     * 
     * @param array $options 입력된 옵션 값
     * @return array 정제된 옵션 값
     */
    public function sanitize_options($options) {
        // 기존 옵션 가져오기
        $existing_options = get_option('wp_kakao_login_options', array());
        
        // 기본 설정 정제
        if (isset($options['kakao_client_id'])) {
            $options['kakao_client_id'] = sanitize_text_field($options['kakao_client_id']);
        }
        
        if (isset($options['kakao_client_secret'])) {
            $options['kakao_client_secret'] = sanitize_text_field($options['kakao_client_secret']);
        }
        
        if (isset($options['kakao_redirect_uri'])) {
            $options['kakao_redirect_uri'] = esc_url_raw($options['kakao_redirect_uri']);
        }
        
        if (isset($options['login_button_text'])) {
            $options['login_button_text'] = sanitize_text_field($options['login_button_text']);
        }
        
        // 고급 설정 정제
        if (isset($options['debug_mode'])) {
            $options['debug_mode'] = (bool)$options['debug_mode'];
        }
        
        // 기존 옵션과 병합하여 누락된 부분 보완
        return array_merge($existing_options, $options);
    }
    
    /**
     * 관리자 페이지 렌더링
     */
    public function render_admin_page() {
        // 관리자 권한 확인
        if (!current_user_can('manage_options')) {
            return;
        }
        
        // 현재 활성 탭 설정
        $active_tab = isset($_GET['tab']) ? sanitize_text_field($_GET['tab']) : 'settings';
        
        ?>
        <div class="wrap">
            <h1><?php echo esc_html(get_admin_page_title()); ?></h1>
            
            <h2 class="nav-tab-wrapper">
                <a href="?page=wp-kakao-login&tab=settings" class="nav-tab <?php echo $active_tab == 'settings' ? 'nav-tab-active' : ''; ?>">
                    <?php _e('설정', 'wp-kakao-login'); ?>
                </a>
                <a href="?page=wp-kakao-login&tab=guide" class="nav-tab <?php echo $active_tab == 'guide' ? 'nav-tab-active' : ''; ?>">
                    <?php _e('가이드', 'wp-kakao-login'); ?>
                </a>
                <a href="?page=wp-kakao-login&tab=logs" class="nav-tab <?php echo $active_tab == 'logs' ? 'nav-tab-active' : ''; ?>">
                    <?php _e('로그', 'wp-kakao-login'); ?>
                </a>
            </h2>
            
            <div class="tab-content">
                <?php
                // 활성 탭에 따라 내용 렌더링
                if ($active_tab == 'settings') {
                    $this->render_settings_tab();
                } elseif ($active_tab == 'guide') {
                    $this->render_guide_tab();
                } elseif ($active_tab == 'logs') {
                    $this->render_logs_tab();
                }
                ?>
            </div>
        </div>
        <?php
    }
    
    /**
     * 설정 탭 렌더링
     */
    private function render_settings_tab() {
        ?>
        <form method="post" action="options.php">
            <?php
            // 설정 필드 출력
            settings_fields('wp_kakao_login_options');
            do_settings_sections('wp_kakao_login');
            submit_button();
            ?>
        </form>
        <?php
    }
    
    /**
     * 가이드 탭 렌더링
     */
    private function render_guide_tab() {
        // 플러그인 URL 및 콜백 URL 가져오기
        $plugin_url = plugin_dir_url(WP_KAKAO_LOGIN_PLUGIN_FILE);
        $direct_callback_url = $plugin_url . 'direct-kakao-callback.php';
        ?>
        <div class="kakao-login-guide">
            <h2><?php _e('카카오 로그인 설정 가이드', 'wp-kakao-login'); ?></h2>
            
            <div class="kakao-guide-section">
                <h3><?php _e('1. 카카오 개발자 애플리케이션 등록', 'wp-kakao-login'); ?></h3>
                <ol>
                    <li><?php _e('카카오 개발자 사이트(<a href="https://developers.kakao.com/" target="_blank">https://developers.kakao.com/</a>)에 접속합니다.', 'wp-kakao-login'); ?></li>
                    <li><?php _e('로그인 후 "내 애플리케이션" 메뉴로 이동합니다.', 'wp-kakao-login'); ?></li>
                    <li><?php _e('"애플리케이션 추가하기" 버튼을 클릭합니다.', 'wp-kakao-login'); ?></li>
                    <li><?php _e('앱 이름과 회사명을 입력하고 "저장" 버튼을 클릭합니다.', 'wp-kakao-login'); ?></li>
                </ol>
            </div>
            
            <div class="kakao-guide-section">
                <h3><?php _e('2. 플랫폼 설정', 'wp-kakao-login'); ?></h3>
                <ol>
                    <li><?php _e('생성된 애플리케이션의 "플랫폼" 탭으로 이동합니다.', 'wp-kakao-login'); ?></li>
                    <li><?php _e('"Web 플랫폼 등록" 버튼을 클릭합니다.', 'wp-kakao-login'); ?></li>
                    <li><?php _e('사이트 도메인에 웹사이트 주소를 입력합니다. (예: https://example.com)', 'wp-kakao-login'); ?></li>
                    <li><?php _e('"등록하기" 버튼을 클릭합니다.', 'wp-kakao-login'); ?></li>
                </ol>
            </div>
            
            <div class="kakao-guide-section">
                <h3><?php _e('3. 카카오 로그인 활성화', 'wp-kakao-login'); ?></h3>
                <ol>
                    <li><?php _e('"카카오 로그인" 탭으로 이동합니다.', 'wp-kakao-login'); ?></li>
                    <li><?php _e('활성화 설정을 "ON"으로 변경합니다.', 'wp-kakao-login'); ?></li>
                    <li><?php _e('동의 항목에서 필요한 항목을 설정합니다. (기본적으로 "닉네임"과 "이메일"을 권장합니다.)', 'wp-kakao-login'); ?></li>
                    <li><?php _e('Redirect URI에 다음 주소를 추가합니다:', 'wp-kakao-login'); ?></li>
                    <li><code><?php echo esc_html($direct_callback_url); ?></code></li>
                    <li><?php _e('"저장" 버튼을 클릭합니다.', 'wp-kakao-login'); ?></li>
                </ol>
                <div class="notice notice-warning">
                    <p><strong><?php _e('중요 안내:', 'wp-kakao-login'); ?></strong> <?php _e('이 플러그인은 워드프레스 리라이트 규칙 대신 직접 접근 방식을 사용합니다. Redirect URI는 반드시 아래 주소를 정확하게 입력해주세요.', 'wp-kakao-login'); ?></p>
                </div>
            </div>
            
            <div class="kakao-guide-section">
                <h3><?php _e('4. API 키 및 시크릿 확인', 'wp-kakao-login'); ?></h3>
                <ol>
                    <li><?php _e('"요약 정보" 탭으로 이동합니다.', 'wp-kakao-login'); ?></li>
                    <li><?php _e('"REST API 키"를 확인하고 복사합니다. (이 키는 이 플러그인의 "카카오 REST API 키" 필드에 입력해야 합니다.)', 'wp-kakao-login'); ?></li>
                    <li><?php _e('"Client Secret" 코드를 활성화하고 코드를 복사합니다. (이 코드는 이 플러그인의 "카카오 Client Secret" 필드에 입력해야 합니다.)', 'wp-kakao-login'); ?></li>
                </ol>
            </div>
            
            <div class="kakao-guide-section">
                <h3><?php _e('5. 플러그인 설정', 'wp-kakao-login'); ?></h3>
                <ol>
                    <li><?php _e('이 설정 페이지의 "설정" 탭으로 이동합니다.', 'wp-kakao-login'); ?></li>
                    <li><?php _e('"카카오 REST API 키" 필드에 복사한 REST API 키를 입력합니다.', 'wp-kakao-login'); ?></li>
                    <li><?php _e('"카카오 Client Secret" 필드에 복사한 Client Secret 코드를 입력합니다.', 'wp-kakao-login'); ?></li>
                    <li><?php _e('"리다이렉트 URI" 필드에 다음 주소가 입력되어 있는지 확인합니다:', 'wp-kakao-login'); ?></li>
                    <li><code><?php echo esc_html($direct_callback_url); ?></code></li>
                    <li><?php _e('기타 설정을 원하는 대로 조정합니다.', 'wp-kakao-login'); ?></li>
                    <li><?php _e('"변경 사항 저장" 버튼을 클릭합니다.', 'wp-kakao-login'); ?></li>
                </ol>
            </div>
            
            <div class="kakao-guide-section">
                <h3><?php _e('6. 단축코드 사용하기', 'wp-kakao-login'); ?></h3>
                <p><?php _e('워드프레스 페이지나 포스트에 다음 단축코드를 추가하여 카카오 로그인 버튼을 표시할 수 있습니다:', 'wp-kakao-login'); ?></p>
                <code>[kakao_login_button]</code>
                
                <p><?php _e('버튼 텍스트를 변경하려면 다음과 같이 사용합니다:', 'wp-kakao-login'); ?></p>
                <code>[kakao_login_button text="카카오로 시작하기"]</code>
                
                <p><?php _e('로그인 후 특정 페이지로 리다이렉트하려면 다음과 같이 사용합니다:', 'wp-kakao-login'); ?></p>
                <code>[kakao_login_button redirect="https://example.com/welcome"]</code>
            </div>
            
            <div class="kakao-guide-section">
                <h3><?php _e('7. 주의사항', 'wp-kakao-login'); ?></h3>
                <ul>
                    <li><?php _e('카카오 개발자 사이트에서 등록한 Redirect URI와 플러그인이 사용하는 URL이 정확히 일치해야 합니다.', 'wp-kakao-login'); ?></li>
                    <li><?php _e('웹사이트가 HTTPS를 사용하는 경우, 카카오 개발자 사이트에 등록하는 모든 URL에도 HTTPS를 사용해야 합니다.', 'wp-kakao-login'); ?></li>
                    <li><?php _e('이 플러그인은 직접 접근 방식을 사용하므로, 플러그인 디렉토리 경로가 변경되면 기능이 작동하지 않을 수 있습니다.', 'wp-kakao-login'); ?></li>
                    <li><?php _e('처음 설정 후 로그인이 작동하지 않는 경우, 로그 탭에서 오류 메시지를 확인하세요.', 'wp-kakao-login'); ?></li>
                </ul>
            </div>
            
            <div class="kakao-guide-section">
                <h3><?php _e('8. 플러그인 파일 구조', 'wp-kakao-login'); ?></h3>
                <p><?php _e('이 플러그인은 다음과 같은 파일 구조를 가지고 있습니다:', 'wp-kakao-login'); ?></p>
                <ul>
                    <li><code>wp-kakao-login.php</code> - 플러그인 메인 파일</li>
                    <li><code>direct-kakao-login.php</code> - 카카오 로그인 직접 접근 페이지</li>
                    <li><code>direct-kakao-callback.php</code> - 카카오 콜백 직접 처리 페이지</li>
                    <li><code>includes/class-kakao-login.php</code> - 카카오 로그인 핵심 클래스</li>
                    <li><code>includes/functions.php</code> - 유틸리티 함수 모음</li>
                    <li><code>includes/admin/class-kakao-admin.php</code> - 관리자 설정 페이지 클래스</li>
                    <li><code>assets/css/kakao-login.css</code> - 카카오 로그인 스타일 파일</li>
                    <li><code>assets/js/kakao-login.js</code> - 카카오 로그인 자바스크립트 파일</li>
                </ul>
            </div>
        </div>
        <?php
    }
    
    /**
     * 로그 탭 렌더링
     */
    private function render_logs_tab() {
        // 로그 파일 경로
        $log_file = WP_KAKAO_LOGIN_PLUGIN_DIR . 'logs/debug.log';
        
        // 로그 삭제 처리
        if (isset($_POST['clear_log']) && check_admin_referer('wp_kakao_login_clear_log')) {
            if (file_exists($log_file)) {
                file_put_contents($log_file, '');
                echo '<div class="notice notice-success is-dismissible"><p>' . __('로그가 삭제되었습니다.', 'wp-kakao-login') . '</p></div>';
            }
        }
        
        ?>
        <div class="kakao-login-logs">
            <h2><?php _e('디버그 로그', 'wp-kakao-login'); ?></h2>
            
            <form method="post" action="">
                <?php wp_nonce_field('wp_kakao_login_clear_log'); ?>
                <p>
                    <input type="submit" name="clear_log" class="button button-secondary" value="<?php _e('로그 삭제', 'wp-kakao-login'); ?>">
                </p>
            </form>
            
            <div class="kakao-log-viewer">
                <?php
                if (file_exists($log_file)) {
                    $log_content = file_get_contents($log_file);
                    
                    if (empty($log_content)) {
                        echo '<p>' . __('로그가 비어 있습니다.', 'wp-kakao-login') . '</p>';
                    } else {
                        // 로그 행 분리
                        $log_lines = array_filter(explode(PHP_EOL, $log_content));
                        
                        // 최신 행이 위에 오도록 역순 정렬
                        $log_lines = array_reverse($log_lines);
                        
                        // 로그 테이블 출력
                        echo '<table class="widefat fixed">';
                        echo '<thead><tr><th>' . __('시간', 'wp-kakao-login') . '</th><th>' . __('메시지', 'wp-kakao-login') . '</th></tr></thead>';
                        echo '<tbody>';
                        
                        foreach ($log_lines as $line) {
                            // 시간 정보 추출 (로그 형식: [YYYY-MM-DD HH:MM:SS] ...)
                            $time_match = array();
                            preg_match('/\[(.*?)\]/', $line, $time_match);
                            $time = isset($time_match[1]) ? $time_match[1] : '';
                            
                            // 메시지 추출 (첫 번째 대괄호 이후의 텍스트)
                            $message = preg_replace('/\[[^\]]*\]\s*/', '', $line, 1);
                            
                            // 오류 메시지 강조 표시
                            $row_class = (strpos($message, 'ERROR') !== false) ? 'error' : '';
                            
                            echo '<tr class="' . $row_class . '">';
                            echo '<td width="20%">' . esc_html($time) . '</td>';
                            echo '<td>' . esc_html($message) . '</td>';
                            echo '</tr>';
                        }
                        
                        echo '</tbody></table>';
                    }
                } else {
                    echo '<p>' . __('로그 파일이 없습니다.', 'wp-kakao-login') . '</p>';
                }
                ?>
            </div>
        </div>
        <?php
    }
    
    /**
     * 기본 설정 섹션 설명 렌더링
     */
    public function render_general_section() {
        echo '<p>' . __('카카오 로그인을 사용하기 위한 기본 설정을 구성합니다.', 'wp-kakao-login') . '</p>';
    }
    
    /**
     * 고급 설정 섹션 설명 렌더링
     */
    public function render_advanced_section() {
        echo '<p>' . __('고급 사용자를 위한 추가 설정을 구성합니다.', 'wp-kakao-login') . '</p>';
    }
    
    /**
     * 클라이언트 ID 필드 렌더링
     */
    public function render_client_id_field() {
        $options = get_option('wp_kakao_login_options');
        $client_id = isset($options['kakao_client_id']) ? $options['kakao_client_id'] : '';
        
        echo '<input type="text" id="kakao_client_id" name="wp_kakao_login_options[kakao_client_id]" value="' . esc_attr($client_id) . '" class="regular-text">';
        echo '<p class="description">' . __('카카오 개발자 사이트에서 발급받은 REST API 키를 입력하세요.', 'wp-kakao-login') . '</p>';
    }
    
    /**
     * 클라이언트 시크릿 필드 렌더링
     */
    public function render_client_secret_field() {
        $options = get_option('wp_kakao_login_options');
        $client_secret = isset($options['kakao_client_secret']) ? $options['kakao_client_secret'] : '';
        
        // 비밀 필드 및 토글 버튼 추가
        ?>
        <div class="password-field-container" style="display:flex;align-items:center;position:relative;">
            <input type="password" id="kakao_client_secret" name="wp_kakao_login_options[kakao_client_secret]" value="<?php echo esc_attr($client_secret); ?>" class="regular-text">
            <button type="button" id="toggle-client-secret" class="button" style="margin-left:5px;" aria-label="비밀값 표시/숨기기">
                <span class="dashicons dashicons-visibility"></span>
            </button>
        </div>
        <p class="description"><?php _e('카카오 개발자 사이트에서 발급받은 Client Secret 코드를 입력하세요.', 'wp-kakao-login'); ?></p>
        <?php
    }
    
    /**
     * 리다이렉트 URI 필드 렌더링
     */
    public function render_redirect_uri_field() {
        // 플러그인 URL 및 리다이렉트 URI 가져오기
        $plugin_url = plugin_dir_url(WP_KAKAO_LOGIN_PLUGIN_FILE);
        $direct_callback_url = $plugin_url . 'direct-kakao-callback.php';
        
        ?>
        <div class="password-field-container">
            <input type="text" id="kakao_redirect_uri" name="wp_kakao_login_options[kakao_redirect_uri]" value="<?php echo esc_attr($direct_callback_url); ?>" class="regular-text" readonly>
            <button type="button" id="copy-redirect-uri" class="button copy-button">
                <span class="dashicons dashicons-clipboard"></span> 복사
            </button>
        </div>
        <p class="description"><?php _e('카카오 개발자 사이트에 등록할 리다이렉트 URI입니다. 이 값을 복사하여 사용하세요.', 'wp-kakao-login'); ?></p>
        <p class="description"><?php _e('주의: 이 플러그인은 워드프레스 리라이트 규칙 대신 직접 접근 방식을 사용합니다.', 'wp-kakao-login'); ?></p>
        <?php
    }
    
    /**
     * 버튼 텍스트 필드 렌더링
     */
    public function render_button_text_field() {
        $options = get_option('wp_kakao_login_options');
        $button_text = isset($options['login_button_text']) ? $options['login_button_text'] : '카카오 계정으로 로그인';
        
        echo '<input type="text" id="login_button_text" name="wp_kakao_login_options[login_button_text]" value="' . esc_attr($button_text) . '" class="regular-text">';
        echo '<p class="description">' . __('카카오 로그인 버튼에 표시할 텍스트를 입력하세요.', 'wp-kakao-login') . '</p>';
    }
    
    /**
     * 디버그 모드 필드 렌더링
     */
    public function render_debug_mode_field() {
        $options = get_option('wp_kakao_login_options');
        $debug_mode = isset($options['debug_mode']) ? $options['debug_mode'] : false;
        
        echo '<label for="debug_mode">';
        echo '<input type="checkbox" id="debug_mode" name="wp_kakao_login_options[debug_mode]" value="1" ' . checked(1, $debug_mode, false) . '>';
        echo __('디버그 모드 사용 (로그 탭에서 자세한 로그를 확인할 수 있습니다)', 'wp-kakao-login');
        echo '</label>';
    }
}