<?php
/*
 * 파일명: class-kakao-login.php
 * 위치: /wp-content/plugins/wp-kakao-login/includes/
 * 기능: 카카오 로그인 핵심 클래스
 * 작성일: 2025-05-01
 * 수정일: 2025-05-02
 */

// 직접 접근 방지
if (!defined('ABSPATH')) {
    exit;
}

// ===================================
// 카카오 로그인 클래스
// ===================================
class WP_Kakao_Login {
    // 옵션 저장 변수
    private $options;
    
    /**
     * 생성자
     */
    public function __construct() {
        // 옵션 로드
        $this->options = get_option('wp_kakao_login_options', array());
        
        // AJAX 처리 등록
        add_action('wp_ajax_nopriv_kakao_login', array($this, 'process_ajax_login'));
        add_action('wp_ajax_kakao_login', array($this, 'process_ajax_login'));
    }
    
    /**
     * 카카오 로그인 버튼 추가
     */
    public function add_kakao_login_button() {
        // 직접 접근 페이지 URL
        $plugin_url = plugin_dir_url(WP_KAKAO_LOGIN_PLUGIN_FILE);
        $direct_kakao_login_url = $plugin_url . 'direct-kakao-login.php';
        
        // 리다이렉트 항상 홈페이지로 설정
        $redirect_to = home_url();
        
        $redirect_param = '?redirect_to=' . urlencode($redirect_to);
        
        // 버튼 텍스트 가져오기
        $button_text = !empty($this->options["login_button_text"]) ? 
                      $this->options["login_button_text"] : "카카오 계정으로 로그인";
        
        // 버튼 HTML 생성 - 간단한 인라인 스타일 적용
        echo '<p style="text-align:center;margin:10px 0;">';
        echo '<a href="' . esc_url($direct_kakao_login_url . $redirect_param) . '" style="display:inline-block;width:100%;padding:10px 0;background:#FEE500;color:#000;text-align:center;text-decoration:none;font-weight:bold;">';
        echo esc_html($button_text);
        echo '</a>';
        echo '</p>';
    }
    
    /**
     * 카카오 로그인 페이지 로드 (직접 접근 방식으로 대체)
     */
    public function load_kakao_login_page() {
        wp_redirect(plugin_dir_url(WP_KAKAO_LOGIN_PLUGIN_FILE) . 'direct-kakao-login.php');
        exit;
    }
    
    /**
     * 카카오 콜백 처리 (직접 접근 방식으로 대체)
     */
    public function process_callback() {
        wp_redirect(plugin_dir_url(WP_KAKAO_LOGIN_PLUGIN_FILE) . 'direct-kakao-callback.php');
        exit;
    }
    
    /**
     * AJAX 로그인 처리
     */
    public function process_ajax_login() {
        // 보안 검증
        if (!check_ajax_referer('wp_kakao_login_nonce', 'nonce', false)) {
            wp_send_json_error(array('message' => '보안 토큰이 유효하지 않습니다.'));
            exit;
        }
        
        // 액세스 토큰 확인
        if (empty($_POST['access_token'])) {
            wp_send_json_error(array('message' => '액세스 토큰이 누락되었습니다.'));
            exit;
        }
        
        $access_token = sanitize_text_field($_POST['access_token']);
        
        // 사용자 정보 요청
        $user_info = $this->get_user_info($access_token);
        
        if (is_wp_error($user_info)) {
            wp_send_json_error(array('message' => $user_info->get_error_message()));
            exit;
        }
        
        // 사용자 인증/생성
        $result = $this->authenticate_user($user_info);
        
        if (is_wp_error($result)) {
            wp_send_json_error(array('message' => $result->get_error_message()));
            exit;
        }
        
        // 리다이렉트 URL 설정
        $redirect_to = isset($_POST['redirect_to']) ? esc_url_raw($_POST['redirect_to']) : home_url();
        
        // 성공 응답
        wp_send_json_success(array(
            'message' => '로그인 성공!',
            'redirect' => $redirect_to
        ));
    }
    
    /**
     * 사용자 정보 요청
     * 
     * @param string $access_token 액세스 토큰
     * @return array|WP_Error 사용자 정보 또는 에러
     */
    private function get_user_info($access_token) {
        // 요청 URL 및 헤더 설정
        $api_url = 'https://kapi.kakao.com/v2/user/me';
        $args = array(
            'headers' => array(
                'Authorization' => 'Bearer ' . $access_token,
                'Content-Type' => 'application/x-www-form-urlencoded;charset=utf-8'
            ),
            'timeout' => 30
        );
        
        // API 요청
        $response = wp_remote_get($api_url, $args);
        
        // 요청 실패 시 오류 반환
        if (is_wp_error($response)) {
            return $response;
        }
        
        // 응답 코드 확인
        $response_code = wp_remote_retrieve_response_code($response);
        if ($response_code !== 200) {
            return new WP_Error('user_info_request_failed', '사용자 정보 요청 실패: ' . $response_code);
        }
        
        // 응답 처리
        $body = wp_remote_retrieve_body($response);
        $data = json_decode($body, true);
        
        if (empty($data) || !isset($data['id'])) {
            return new WP_Error('invalid_user_info_response', '유효하지 않은 사용자 정보 응답');
        }
        
        return $data;
    }
    
    /**
     * 사용자 인증/생성
     * 
     * @param array $user_info 카카오에서 가져온 사용자 정보
     * @return int|WP_Error 사용자 ID 또는 에러
     */
    private function authenticate_user($user_info) {
        // 카카오 ID 확인
        $kakao_id = isset($user_info['id']) ? $user_info['id'] : '';
        if (empty($kakao_id)) {
            return new WP_Error('missing_kakao_id', '카카오 ID가 없습니다.');
        }
        
        // 카카오 계정 정보
        $kakao_nickname = isset($user_info['properties']['nickname']) ? $user_info['properties']['nickname'] : '';
        $kakao_email = isset($user_info['kakao_account']['email']) ? $user_info['kakao_account']['email'] : '';
        
        // 카카오 ID로 사용자 검색
        $users = get_users(array(
            'meta_key' => 'kakao_id',
            'meta_value' => $kakao_id,
            'number' => 1,
            'count_total' => false
        ));
        
        $user = !empty($users) ? $users[0] : false;
        
        // 기존 사용자가 있는 경우 로그인 처리
        if ($user) {
            wp_set_auth_cookie($user->ID, true);
            do_action('wp_login', $user->user_login, $user);
            return $user->ID;
        }
        
        // 이메일이 있는 경우 해당 이메일로 사용자 검색
        if (!empty($kakao_email)) {
            $user = get_user_by('email', $kakao_email);
            if ($user) {
                // 기존 사용자에게 카카오 ID 연결
                update_user_meta($user->ID, 'kakao_id', $kakao_id);
                wp_set_auth_cookie($user->ID, true);
                do_action('wp_login', $user->user_login, $user);
                return $user->ID;
            }
        }
        
        // 새 사용자 생성
        return $this->create_new_user($kakao_id, $kakao_nickname, $kakao_email);
    }
    
    /**
     * 새 사용자 생성
     * 
     * @param string $kakao_id 카카오 ID
     * @param string $nickname 사용자 닉네임
     * @param string $email 사용자 이메일
     * @return int|WP_Error 사용자 ID 또는 에러
     */
    private function create_new_user($kakao_id, $nickname, $email) {
        // 이메일이 없는 경우 임시 이메일 생성
        if (empty($email)) {
            $email = 'kakao_' . $kakao_id . '@kakao.user';
        }
        
        // 사용자명 생성 (닉네임 + 랜덤 숫자)
        $username = sanitize_user($nickname, true);
        if (empty($username)) {
            $username = 'kakao_user';
        }
        
        // 중복 방지
        if (username_exists($username)) {
            $username .= '_' . substr($kakao_id, -6);
        }
        
        // 랜덤 비밀번호 생성
        $password = wp_generate_password(16, true, true);
        
        // 사용자 생성
        $user_id = wp_create_user($username, $password, $email);
        
        if (is_wp_error($user_id)) {
            return $user_id;
        }
        
        // 사용자 메타 데이터 업데이트
        update_user_meta($user_id, 'kakao_id', $kakao_id);
        update_user_meta($user_id, 'nickname', $nickname);
        
        // 사용자 역할 설정
        $user = new WP_User($user_id);
        $user->set_role('subscriber');
        
        // 인증 쿠키 설정
        wp_set_auth_cookie($user_id, true);
        do_action('wp_login', $username, $user);
        
        return $user_id;
    }
}