<?php
/*
 * 파일명: functions.php
 * 위치: /wp-content/plugins/wp-kakao-login/includes/
 * 기능: 카카오 로그인 플러그인 헬퍼 함수
 * 작성일: 2025-05-01
 * 수정일: 2025-05-01
 */

// 직접 접근 방지
if (!defined('ABSPATH')) {
    exit;
}

// ===================================
// 카카오 로그인 버튼 출력 함수
// ===================================
/**
 * 카카오 로그인 버튼 출력
 * 
 * @param bool $echo 출력 여부 (true: 출력, false: 반환)
 * @param string $text 버튼 텍스트
 * @param string $redirect 리다이렉트 URL
 * @return string|void 출력 여부에 따라 HTML 문자열 반환 또는 출력
 */
function wp_kakao_login_button($echo = true, $text = '', $redirect = '') {
    // 이미 로그인한 사용자는 로그인 버튼 대신 다른 내용 표시
    if (is_user_logged_in()) {
        $current_user = wp_get_current_user();
        $output = '<div class="kakao-login-wrap kakao-already-logged-in">
            <p>' . sprintf(__('안녕하세요, %s님!', 'wp-kakao-login'), $current_user->display_name) . '</p>
            <a href="' . wp_logout_url(home_url()) . '" class="kakao-logout-link">' . __('로그아웃', 'wp-kakao-login') . '</a>
        </div>';
        
        if ($echo) {
            echo $output;
            return;
        }
        return $output;
    }
    
    // 옵션에서 버튼 텍스트 가져오기
    $options = get_option('wp_kakao_login_options', array());
    $button_text = !empty($text) ? 
                   $text : 
                   (!empty($options['login_button_text']) ? $options['login_button_text'] : '카카오 계정으로 로그인');
    
    // 리다이렉트 URL 설정
    $redirect_url = !empty($redirect) ? esc_url($redirect) : '';
    
    // HTML 출력 버퍼링 시작
    ob_start();
    ?>
    <div class="kakao-login-wrap">
        <div class="input-group">
            <span class="input-group-text bg-white border-end-0">
                <i class="bi bi-chat-fill"></i>
            </span>
            <button type="button" class="kakao-login-btn form-control border-start-0" 
                    data-redirect="<?php echo esc_attr($redirect_url); ?>">
                <?php echo esc_html($button_text); ?>
            </button>
        </div>
    </div>
    <?php
    
    // 버퍼 내용 가져오기
    $output = ob_get_clean();
    
    // 출력 여부에 따라 출력 또는 반환
    if ($echo) {
        echo $output;
        return;
    }
    return $output;
}

// ===================================
// 로그 기록 함수
// ===================================
/**
 * 디버그 로그 기록
 * 
 * @param string $message 로그 메시지
 * @param mixed $data 추가 데이터 (옵션)
 * @param string $level 로그 레벨 (info, warning, error)
 */
function wp_kakao_login_log($message, $data = null, $level = 'info') {
    // 디버그 모드 확인
    $debug_enabled = apply_filters('wp_kakao_login_debug_enabled', true);
    
    if (!$debug_enabled) {
        return;
    }
    
    // 로그 파일 경로
    $log_file = WP_KAKAO_LOGIN_PLUGIN_DIR . 'logs/debug.log';
    
    // 로그 디렉토리 확인 및 생성
    $log_dir = dirname($log_file);
    if (!file_exists($log_dir)) {
        wp_mkdir_p($log_dir);
    }
    
    // 데이터가 있는 경우 JSON으로 변환
    $data_string = '';
    if ($data !== null) {
        if (is_array($data) || is_object($data)) {
            $data_string = ' - Data: ' . json_encode($data, JSON_UNESCAPED_UNICODE);
        } else {
            $data_string = ' - Data: ' . $data;
        }
    }
    
    // 로그 레벨 색상 코드 (콘솔 출력용)
    $level_map = array(
        'info' => 'INFO',
        'warning' => 'WARNING',
        'error' => 'ERROR'
    );
    
    $level_text = isset($level_map[$level]) ? $level_map[$level] : 'INFO';
    
    // 메시지 포맷팅
    $log_message = '[' . date('Y-m-d H:i:s') . '] [' . $level_text . '] ' . $message . $data_string . PHP_EOL;
    
    // 로그 기록
    error_log($log_message, 3, $log_file);
}

// ===================================
// 옵션 관련 함수
// ===================================
/**
 * 카카오 로그인 옵션 가져오기
 * 
 * @param string $key 옵션 키
 * @param mixed $default 기본값
 * @return mixed 옵션 값
 */
function wp_kakao_login_get_option($key, $default = '') {
    $options = get_option('wp_kakao_login_options', array());
    
    if (isset($options[$key])) {
        return $options[$key];
    }
    
    return $default;
}

/**
 * 카카오 로그인 옵션 업데이트
 * 
 * @param string $key 옵션 키
 * @param mixed $value 옵션 값
 * @return bool 성공 여부
 */
function wp_kakao_login_update_option($key, $value) {
    $options = get_option('wp_kakao_login_options', array());
    $options[$key] = $value;
    
    return update_option('wp_kakao_login_options', $options);
}

// ===================================
// 카카오 API 관련 함수
// ===================================
/**
 * 카카오 인증 URL 생성
 * 
 * @param string $redirect_uri 리다이렉트 URI
 * @param string $state 상태 파라미터
 * @return string 카카오 인증 URL
 */
function wp_kakao_login_get_auth_url($redirect_uri = '', $state = '') {
    $client_id = wp_kakao_login_get_option('kakao_client_id', '');
    
    if (empty($client_id)) {
        return '';
    }
    
    if (empty($redirect_uri)) {
        $redirect_uri = wp_kakao_login_get_option('kakao_redirect_uri', home_url('/kakao-callback'));
    }
    
    $auth_url = 'https://kauth.kakao.com/oauth/authorize';
    $auth_url .= '?client_id=' . urlencode($client_id);
    $auth_url .= '&redirect_uri=' . urlencode($redirect_uri);
    $auth_url .= '&response_type=code';
    
    if (!empty($state)) {
        $auth_url .= '&state=' . urlencode($state);
    }
    
    return $auth_url;
}

/**
 * 카카오 로그인 URL 생성
 * 
 * @param string $redirect_to 로그인 후 리다이렉트할 URL
 * @return string 카카오 로그인 URL
 */
function wp_kakao_login_get_login_url($redirect_to = '') {
    $login_url = home_url('/kakao-login');
    
    if (!empty($redirect_to)) {
        $login_url = add_query_arg('redirect_to', urlencode($redirect_to), $login_url);
    }
    
    return $login_url;
}

/**
 * 카카오 프로필 이미지 URL 가져오기
 * 
 * @param int $user_id 사용자 ID
 * @return string 프로필 이미지 URL
 */
function wp_kakao_login_get_profile_image($user_id) {
    $profile_image = get_user_meta($user_id, 'kakao_profile_image', true);
    
    if (!empty($profile_image)) {
        return $profile_image;
    }
    
    // 기본 Gravatar 이미지 반환
    return get_avatar_url($user_id);
}

// ===================================
// 워드프레스 통합 관련 함수
// ===================================
/**
 * 로그인 URL 필터 - 카카오 로그인 지원
 * 
 * @param string $login_url 기존 로그인 URL
 * @param string $redirect 리다이렉트 URL
 * @param bool $force_reauth 강제 재인증 여부
 * @return string 수정된 로그인 URL
 */
function wp_kakao_login_filter_login_url($login_url, $redirect, $force_reauth) {
    // 카카오 로그인 사용 설정 확인
    $use_kakao_login_page = apply_filters('wp_kakao_login_use_login_page', false);
    
    if ($use_kakao_login_page) {
        $login_url = wp_kakao_login_get_login_url($redirect);
    }
    
    return $login_url;
}

// 로그인 URL 필터 등록 (선택적)
// add_filter('login_url', 'wp_kakao_login_filter_login_url', 10, 3);

/**
 * 사용자 아바타 필터 - 카카오 프로필 이미지 사용
 * 
 * @param string $avatar HTML 아바타 이미지 태그
 * @param mixed $id_or_email 사용자 ID, 이메일 또는 객체
 * @param int $size 아바타 크기
 * @param string $default 기본 아바타 유형
 * @param string $alt 대체 텍스트
 * @return string 수정된 아바타 HTML
 */
function wp_kakao_login_filter_avatar($avatar, $id_or_email, $size, $default, $alt) {
    $user_id = 0;
    
    if (is_numeric($id_or_email)) {
        $user_id = (int)$id_or_email;
    } elseif (is_string($id_or_email)) {
        $user = get_user_by('email', $id_or_email);
        if ($user) {
            $user_id = $user->ID;
        }
    } elseif (is_object($id_or_email)) {
        if (isset($id_or_email->user_id)) {
            $user_id = (int)$id_or_email->user_id;
        } elseif (isset($id_or_email->ID)) {
            $user_id = (int)$id_or_email->ID;
        }
    }
    
    if ($user_id > 0) {
        $kakao_profile_image = get_user_meta($user_id, 'kakao_profile_image', true);
        
        if (!empty($kakao_profile_image)) {
            $avatar = "<img alt='" . esc_attr($alt) . "' src='" . esc_url($kakao_profile_image) . "' class='avatar avatar-{$size} photo' height='{$size}' width='{$size}' />";
        }
    }
    
    return $avatar;
}

// 아바타 필터 등록 (선택적)
// add_filter('get_avatar', 'wp_kakao_login_filter_avatar', 10, 5);

/**
 * 사용자 정보 동기화
 * 
 * @param int $user_id 사용자 ID
 * @param array $kakao_info 카카오 사용자 정보
 * @return bool 성공 여부
 */
function wp_kakao_login_sync_user_data($user_id, $kakao_info) {
    if (empty($user_id) || empty($kakao_info) || !isset($kakao_info['id'])) {
        return false;
    }
    
    // 카카오 ID 저장
    update_user_meta($user_id, 'kakao_id', $kakao_info['id']);
    
    // 카카오 프로필 정보가 있는 경우 저장
    if (isset($kakao_info['properties'])) {
        $properties = $kakao_info['properties'];
        
        if (isset($properties['nickname'])) {
            update_user_meta($user_id, 'nickname', $properties['nickname']);
        }
        
        if (isset($properties['profile_image'])) {
            update_user_meta($user_id, 'kakao_profile_image', $properties['profile_image']);
        }
        
        if (isset($properties['thumbnail_image'])) {
            update_user_meta($user_id, 'kakao_thumbnail_image', $properties['thumbnail_image']);
        }
    }
    
    // 카카오 계정 정보가 있는 경우 저장
    if (isset($kakao_info['kakao_account'])) {
        $account = $kakao_info['kakao_account'];
        
        if (isset($account['email']) && $account['email']) {
            $user_email = get_userdata($user_id)->user_email;
            $kakao_email = $account['email'];
            
            // 카카오 이메일이 있고, 현재 사용자의 이메일이 임시 이메일인 경우 업데이트
            if (!empty($kakao_email) && strpos($user_email, '@kakao.user') !== false) {
                wp_update_user(array(
                    'ID' => $user_id,
                    'user_email' => $kakao_email
                ));
            }
            
            update_user_meta($user_id, 'kakao_email', $kakao_email);
        }
        
        if (isset($account['profile']) && isset($account['profile']['nickname'])) {
            update_user_meta($user_id, 'nickname', $account['profile']['nickname']);
        }
        
        // 성별 정보
        if (isset($account['gender']) && $account['gender']) {
            update_user_meta($user_id, 'kakao_gender', $account['gender']);
        }
        
        // 연령대 정보
        if (isset($account['age_range']) && $account['age_range']) {
            update_user_meta($user_id, 'kakao_age_range', $account['age_range']);
        }
        
        // 생일 정보
        if (isset($account['birthday']) && $account['birthday']) {
            update_user_meta($user_id, 'kakao_birthday', $account['birthday']);
        }
    }
    
    // 동기화 시간 기록
    update_user_meta($user_id, 'kakao_last_sync', current_time('mysql'));
    
    return true;
}